# opkbuild
# lib/locale.sh
# Locale functions.
#
# Copyright (C) 2012 Patrick "P. J." McDermott
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

[ x"${_OB_LOCALE_SM+set}" = x'set' ] && return 0
_OB_LOCALE_SM=1

ob_use output

_OB_DEFAULT_LOCALE='en_US'
_OB_INTERNAL_TEXT_DOMAIN='libopkbuild_1'
_OB_TEXT_DOMAIN=
_OB_LOCALE_PATH='@@LOCALEDIR@@/%s/LC_MESSAGES/%s.ms'

ob_get_text_domain()
{
	if [ ${#} -ne 0 ]; then
		return 125
	fi

	echo "${_OB_TEXT_DOMAIN}"
	return 0
}

ob_set_text_domain()
{
	if [ ${#} -eq 1 ]; then
		_OB_TEXT_DOMAIN="${1}"
	else
		return 125
	fi

	# Make sure LC_MESSAGES is set.
	if [ -z "${LC_MESSAGES}" ]; then
		if [ -n "${LC_ALL}" ]; then
			LC_MESSAGES="${LC_ALL}"
		elif [ -n "${LANG}" ]; then
			LC_MESSAGES="${LANG}"
		else
			LC_MESSAGES="${_OB_DEFAULT_LOCALE}"
		fi
	fi

	# TODO: Exit early if domain's messages have already been loaded.

	# Try to load the messages.
	if ! _ob_try_load_messages "${LC_MESSAGES%.*}"; then
		if ! _ob_try_load_messages "${LC_MESSAGES%_*}"; then
			if ! _ob_try_load_messages "${_OB_DEFAULT_LOCALE}"; then
				ob_warn 'Cannot load messages in "%s" domain.' \
					"${_OB_TEXT_DOMAIN}"
				return 1
			fi
		fi
	fi

	return 0
}

ob_set_locale_path()
{
	local format=

	if [ ${#} -eq 1 ]; then
		format="${1}"
	else
		return 125
	fi

	_OB_LOCALE_PATH="${format}"

	return 0
}

ob_get_msg()
{
	local msgid=

	if [ ${#} -eq 1 ]; then
		msgid="${1}"
	else
		return 125
	fi

	IFS= eval echo \$\{"msg_${_OB_TEXT_DOMAIN}_${msgid}"\}

	return 0
}

_ob_try_load_messages()
{
	local locale=
	local ms=

	locale="${1}"
	ms="$(printf "${_OB_LOCALE_PATH}" \
		"${locale}" "${_OB_TEXT_DOMAIN}")"

	if [ -f "${ms}" ]; then
		. "${ms}"
	else
		return 1
	fi

	return 0
}
